# -*- coding: utf-8 -*-
import sys

from os.path import join, isdir

from django.conf.urls import patterns, url

from djangojs.conf import settings
from djangojs.views import UrlsJsonView, ContextJsonView

js_info_dict = {
    'packages': [],
}

for app in settings.INSTALLED_APPS:
    module = sys.modules[app]
    for path in module.__path__:
        if isdir(join(path, 'locale')):
            js_info_dict['packages'].append(app)

urlpatterns = patterns('',
    url(r'^urls$', UrlsJsonView.as_view(), name='django_js_urls'),
    url(r'^context$', ContextJsonView.as_view(), name='django_js_context'),
    url(r'^trans$', 'django.views.i18n.javascript_catalog', js_info_dict, name='js_catalog'),
)



# -*- coding: utf-8 -*-
import sys

# Default configuration values for Django.js
# All values used by Django.js needs to appears here.
DEFAULTS = {
    'DEBUG': True,
    'TESTING': 'test' in sys.argv,
}

# All required settings keys to import rom project
REQUIREDS = (
    'INSTALLED_APPS',
    'STATIC_URL',
    'ROOT_URLCONF',
)


class Settings(dict):
    '''
    Django settings wrapper
    '''

    def __init__(self):
        """
        Try and initialize with Django settings.
        All values from DEFAULTS will be overwritten by project settings if present.
        """
        try:
            from django.conf import settings

            self.update(settings._wrapped.__dict__)

            # Warn for missings settings
            for key in REQUIREDS:
                if not key in self:
                    raise Exception('%s settings key is mandatory')

            # Set defaults for missing keys
            for key, value in DEFAULTS.iteritems():
                if not key in self:
                    self[key] = value

        except ImportError:
            pass

    # def set_max_message_length(self):
    #     extra_message_chars = "#PRIVMSG %s :\r\n" % self["IRC_CHANNEL"]
    #     self["MAX_MESSAGE_LENGTH"] = 512 - len(extra_message_chars)

    def __getattr__(self, k):
        return self[k]


settings = Settings()



# -*- coding: utf-8 -*-
import re
import sys

from os.path import join, dirname
from subprocess import Popen, STDOUT, PIPE
from tempfile import NamedTemporaryFile
# from collections import OrderedDict

from django.core.urlresolvers import reverse
from django.test import LiveServerTestCase

# TAP regex
TAP_MODULE_REGEX = re.compile(r'^(?P<indent>\s*)# module: (?P<name>.*)$')
TAP_TEST_REGEX = re.compile(r'^(?P<indent>\s*)# test: (?P<name>.*)$')
TAP_ASSERTION_REGEX = re.compile(r'^(?P<indent>\s*)(?P<type>(?:not )?ok) (?P<num>\d+)(?: - (?P<details>.*))?$')
TAP_STACK_REGEX = re.compile(r'^(?P<indent>\s*)#(?:\s+)at(?P<stack>.*)$')
TAP_END_REGEX = re.compile(r'^(?P<indent>\s*)(?P<start>\d+)\.\.(?P<end>\d+)(?: - (?P<details>.*))?$')
TAP_DETAILS_REGEX = re.compile(r'^(?:expected: (?P<expected>.*), got: (?P<got>.*), )?(?:matcher: (?P<matcher>.*), )?source: at (?P<source>.*)$')

# Output format
LINE_SIZE = 70
INDENT = 4
DEFAULT_ENCODING = 'utf-8'
COLORS = {
      "green": 32,
      "red": 31,
      "yellow": 33
}


def colorize(text, color):
    return u'\033[%sm%s\033[0m' % (COLORS[color], text)


def parse_verbosity():
    verbosity = 1
    regex = re.compile(r'(?:-v|--verbosity)(?:(?:\=)?([0-3]))?')
    for idx, arg in enumerate(sys.argv):
        match = regex.match(arg)
        if match:
            verbosity = int(match.group(1)) if match.group(1) else int(sys.argv[idx + 1])
            break
    return verbosity

VERBOSITY = parse_verbosity()
VERBOSE = VERBOSITY > 1


class TapItem(object):
    parent = None
    parsed_indent = ''

    @property
    def indent(self):
        if self.parent and isinstance(self.parent, TapItem):
            if isinstance(self.parent, TapModule):
                return self.parent.indent + ' ' * INDENT
            else:
                return self.parent.indent
        return ''


class TapGroup(list, TapItem):
    def __init__(self, name='', parent=None, parsed_indent='', *args, **kwargs):
        super(TapGroup, self).__init__(*args, **kwargs)
        self.name = name
        self.parent = parent
        self.parsed_indent = parsed_indent

    def __unicode__(self):
        return u'%s: %s' % (self.name, super(TapGroup, self).__unicode__())

    def __str__(self):
        return unicode(self).encode(sys.stdout.encoding or DEFAULT_ENCODING, 'replace')

    def __nonzero__(self):
        return True

    def append(self, item):
        if isinstance(item, (TapGroup, TapAssertion)) and not item.parent:
            item.parent = self
        super(TapGroup, self).append(item)


class TapModule(TapGroup):

    def display(self):
        return u'%s%s' % (self.indent, self.name)

    def __unicode__(self):
        return u'# module: %s' % self.name

    @classmethod
    def parse(cls, line):
        match = TAP_MODULE_REGEX.match(line.rstrip())
        if match:
            return cls(
                match.group('name').strip(),
                parsed_indent=match.group('indent')
            )
        else:
            return None


class TapTest(TapGroup):

    def display(self):
        assertions = [result.display(True) for result in self]
        if assertions:
            return u'%s%s (%s)' % (self.indent, self.name, ' '.join(assertions))
        else:
            return u'%s%s' % (self.indent, self.name)

    def __unicode__(self):
        return u'# test: %s' % self.name

    @classmethod
    def parse(cls, line):
        match = TAP_TEST_REGEX.match(line.rstrip())
        if match:
            return cls(match.group('name').strip(), parsed_indent=match.group('indent'))
        else:
            return None


class TapAssertion(TapItem):
    def __init__(self, num, success=True, msg=None, parsed_indent='', *args, **kwargs):
        super(TapAssertion, self).__init__()
        self.num = num
        self.success = success
        self.msg = msg
        self.expected = None
        self.got = None
        self.matcher = None
        self.stack = None
        self.parsed_indent = parsed_indent

    def display(self, inline=False):
        if inline:
            return colorize(u'ok', 'green') if self.success else colorize(u'ko', 'red')
        else:
            text = self.indent
            if self.success:
                text += colorize(u'ok %s' % self.num, 'green')
            else:
                text += colorize(u'not ok %s' % self.num, 'red')
            text = '%s - %s' % (text, self.msg) if self.msg else text
            if self.expected is not None and self.got is not None:
                text = '\n'.join([text, '# expected: %s' % self.expected, '# got: %s' % self.got])
            if self.stack:
                text = '\n'.join([text, '\n'.join(['# stack: %s' % line for line in self.stack])])
            return text

    def __unicode__(self):
        return u'ok %s' % self.num if self.success else u'not ok %s' % self.num

    def __str__(self):
        return unicode(self).encode(sys.stdout.encoding or DEFAULT_ENCODING, 'replace')

    @classmethod
    def parse(cls, line):
        match = TAP_ASSERTION_REGEX.match(line.rstrip())
        if match:
            assertion = TapAssertion(
                match.group('num'),
                match.group('type') == 'ok',
                parsed_indent=match.group('indent')
            )
            if match.group('details'):
                details_match = TAP_DETAILS_REGEX.match(match.group('details'))
                if details_match and details_match.group('expected'):
                    assertion.expected = details_match.group('expected')
                    assertion.got = details_match.group('got')
                if details_match and details_match.group('matcher'):
                    assertion.matcher = details_match.group('matcher')
                if details_match:
                    assertion.stack = [details_match.group('source')]
                else:
                    assertion.msg = match.group('details')
            return assertion
        else:
            return None


HIERARCHY = (
    TapModule,
    TapTest,
    TapAssertion,
)


def hierarchy(item):
    if not isinstance(item, TapItem):
        raise ValueError('Item should be a TapItem')
    return HIERARCHY.index(item.__class__)


class TapOutputParser(object):
    def __init__(self, proc, yield_class=TapTest):
        if not issubclass(yield_class, TapItem):
            raise ValueError('yield class should extend TapItem')
        self.proc = proc
        self.suites = TapGroup()
        self.current = self.suites
        self.yield_class = yield_class

    def parse(self):
        while self.proc.poll() is None:
            for item in self.parse_line(self.proc.stdout.readline()):
                yield item

        for item in self.get_lasts():
            yield item

    def parse_line(self, line):
        item = TapModule.parse(line) or TapTest.parse(line) or TapAssertion.parse(line)
        if item is not None:
            return self.set_current(item)

        match = TAP_STACK_REGEX.match(line.rstrip())
        if match:
            self.current.stack.append(match.group('stack'))
            return []

        match = TAP_END_REGEX.match(line.rstrip())
        if match and VERBOSITY > 2:
            print '# end %s-%s' % (match.group('start'), match.group('end'))
            return []

        if line and VERBOSITY > 2:
            print 'not matched: %s' % line

        return []

    def set_current(self, item=None):
        if item and not isinstance(item, TapItem):
            raise ValueError('Should be a TAP item')

        ended = []

        while self.current.parent and hierarchy(item) < hierarchy(self.current):
            if isinstance(self.current, self.yield_class):
                ended.append(self.current)
            self.current = self.current.parent

        while self.current.parent \
                and hierarchy(item) == hierarchy(self.current) \
                and len(item.parsed_indent) <= len(self.current.parsed_indent):
            if isinstance(self.current, self.yield_class):
                ended.append(self.current)
            self.current = self.current.parent

        self.current.append(item)
        self.current = item

        if hierarchy(self.current) < HIERARCHY.index(self.yield_class):
            ended.append(self.current)

        return ended

    def get_lasts(self):
        lasts = []
        while isinstance(self.current, TapItem) and self.current.parent:
            if isinstance(self.current, self.yield_class):
                lasts.append(self.current)
            self.current = self.current.parent
        return lasts


class JsTestCase(LiveServerTestCase):
    '''
    Test helper to run JS test (Jasmine/Qunit) with phantomjs
    '''

    def phantomjs(self, *args, **kwargs):
        separator = '=' * LINE_SIZE
        title = kwargs['title'] if 'title' in kwargs else 'phantomjs output'
        nb_spaces = (LINE_SIZE - len(title)) / 2

        if VERBOSE:
            print ''
            print separator
            print ' ' * nb_spaces + title
            print separator

        with NamedTemporaryFile(delete=True) as cookies_file:
            cmd = ('phantomjs', '--cookies-file=%s' % cookies_file.name) + args
            proc = Popen(cmd, stdout=PIPE, stderr=STDOUT, universal_newlines=True)
            parser = TapOutputParser(proc)

            for item in parser.parse():
                if VERBOSE:
                    print item.display()

        if VERBOSE:
            print separator

        self.assertEqual(proc.returncode, 0)

    def run_jasmine(self, runner_url_name, title='Jasmine test suite'):
        jasmine_runner_url = ''.join([self.live_server_url, reverse(runner_url_name)])
        phantomjs_jasmine_runner = join(dirname(__file__), 'phantomjs', 'jasmine-runner.js')

        return self.phantomjs(phantomjs_jasmine_runner, jasmine_runner_url, title=title)

    def run_qunit(self, runner_url_name, title='Qunit test suite'):
        qunit_runner_url = ''.join([self.live_server_url, reverse(runner_url_name)])
        phantomjs_qunit_runner = join(dirname(__file__), 'phantomjs', 'qunit-runner.js')  # 'run-qunit.js')

        return self.phantomjs(phantomjs_qunit_runner, qunit_runner_url, title=title)



# -*- coding: utf-8 -*-
import json
import logging
import os
import re
import sys
import types

from django.contrib.staticfiles import finders
from django.contrib.staticfiles.utils import matches_patterns
from django.core.serializers.json import DjangoJSONEncoder
from django.core.urlresolvers import RegexURLPattern, RegexURLResolver
from django.http import HttpResponse
from django.template.context import RequestContext
from django.utils import translation
from django.utils.datastructures import SortedDict
from django.views.generic import View, TemplateView

from djangojs.conf import settings

logger = logging.getLogger(__name__)


__all__ = (
    'UrlsJsonView',
    'ContextJsonView',
    'JsTestView',
    'JasmineView',
    'QUnitView',
)

RE_KWARG = re.compile(r"(\(\?P\<(.*?)\>.*?\))")  # Pattern for recongnizing named parameters in urls
RE_ARG = re.compile(r"(\(.*?\))")  # Pattern for recognizing unnamed url parameters
RE_OPT = re.compile(r"\w\?")  # Pattern for recognizing optionnal character
RE_OPT_GRP = re.compile(r"\(\?\:.*\)\?")  # Pattern for recognizing optionnal group


class JsonView(View):
    '''
    List all registered URLs as a JSON object.
    '''
    def get(self, request, *args, **kwargs):
        data = self.get_context_data(request, *args, **kwargs)
        return HttpResponse(
            json.dumps(data, cls=DjangoJSONEncoder),
            mimetype="application/json"
        )


class UrlsJsonView(JsonView):
    '''
    List all registered URLs as a JSON object.
    '''
    def get_context_data(self, request, *args, **kwargs):
        if not hasattr(settings, 'ROOT_URLCONF'):
            raise Exception
        module = settings.ROOT_URLCONF
        return self.get_urls(module)

    def get_urls(self, module, prefix=''):
        urls = SortedDict()
        if isinstance(module, (str, unicode)):
            __import__(module)
            root_urls = sys.modules[module]
            patterns = root_urls.urlpatterns
        elif isinstance(module, (list, tuple)):
            patterns = module
        elif isinstance(module, types.ModuleType):
            patterns = module.urlpatterns
        else:
            raise TypeError('Unsupported type: %s' % type(module))

        for pattern in patterns:
            if issubclass(pattern.__class__, RegexURLPattern):
                if pattern.name:
                    full_url = prefix + pattern.regex.pattern
                    for chr in ['^', '$']:
                        full_url = full_url.replace(chr, '')
                    # remove optionnal non capturing groups
                    opt_grp_matches = RE_OPT_GRP.findall(full_url)
                    if opt_grp_matches:
                        for match in opt_grp_matches:
                            full_url = full_url.replace(match, '')
                    # remove optionnal characters
                    opt_matches = RE_OPT.findall(full_url)
                    if opt_matches:
                        for match in opt_matches:
                            full_url = full_url.replace(match, '')
                    # handle kwargs, args
                    kwarg_matches = RE_KWARG.findall(full_url)
                    if kwarg_matches:
                        for el in kwarg_matches:
                            # prepare the output for JS resolver
                            full_url = full_url.replace(el[0], "<%s>" % el[1])
                    # after processing all kwargs try args
                    args_matches = RE_ARG.findall(full_url)
                    if args_matches:
                        for el in args_matches:
                            full_url = full_url.replace(el, "<>")  # replace by a empty parameter name
                    urls[pattern.name] = "/" + full_url
            elif issubclass(pattern.__class__, RegexURLResolver):
                if pattern.urlconf_name:
                    urls.update(self.get_urls(pattern.urlconf_name, pattern.regex.pattern))
        return urls


class ContextJsonView(JsonView):

    SERIALIZERS = {
        'LANGUAGES': 'serialize_languages',
        # 'perms': 'serialize_perms',
    }

    def get_context_data(self, request, *args, **kwargs):
        data = {}
        for context in RequestContext(request):
            for key, value in context.iteritems():
                # print key, value
                if key in self.SERIALIZERS:
                    serializer_name = self.SERIALIZERS[key]
                    if hasattr(self, serializer_name):
                        serializer = getattr(self, serializer_name)
                        data[key] = serializer(value)
                elif isinstance(value, (str, tuple, list, dict, int, bool)):
                    data[key] = value
        if 'LANGUAGE_CODE' in data:
            # Dirty hack to fix non included default
            language_code = 'en' if data['LANGUAGE_CODE'] == 'en-us' else data['LANGUAGE_CODE']
            language = translation.get_language_info(language_code)
            data['LANGUAGE_NAME'] = language['name']
            data['LANGUAGE_NAME_LOCAL'] = language['name_local']
        return data

    def serialize_perms(self, perms):
        return None

    def serialize_languages(self, languages):
        return dict(languages)


class JsTestView(TemplateView):
    '''
    Base class for JS tests views
    '''
    js_files = None
    django_js = False

    def get_context_data(self, **kwargs):
        context = super(JsTestView, self).get_context_data(**kwargs)

        context['js_test_files'] = self.get_js_files()
        context['use_django_js'] = self.django_js

        return context

    def get_js_files(self):
        if self.js_files:
            if isinstance(self.js_files, str):
                matches = lambda path: matches_patterns(path, [self.js_files])
            elif isinstance(self.js_files, (list, tuple)):
                matches = lambda path: matches_patterns(path, self.js_files)
            return [path for path in self.get_static_files() if matches(path)]
        return []

    def get_static_files(self):
        files = []
        for finder in finders.get_finders():
            for path, storage in finder.list(None):
                # Prefix the relative path if the source storage contains it
                if getattr(storage, 'prefix', None):
                    prefixed_path = os.path.join(storage.prefix, path)
                else:
                    prefixed_path = path

                if prefixed_path not in files:
                    files.append(prefixed_path)
        return files


class JasmineView(JsTestView):
    '''
    Render a Jasmine test runner.
    '''
    template_name = 'djangojs/jasmine-runner.html'


class QUnitView(JsTestView):
    '''
    Render a QUnit test runner
    '''
    template_name = 'djangojs/qunit-runner.html'



import json

from django.core.urlresolvers import reverse
from django.template import Context, Template
from django.test import TestCase
from django.test.utils import override_settings
from django.conf import global_settings
from django.utils import translation

from djangojs.conf import settings
from djangojs.runners import JsTestCase
from djangojs.views import JsTestView


def custom_processor(request):
    return {'CUSTOM': 'CUSTOM_VALUE'}


@override_settings(
    TEMPLATE_CONTEXT_PROCESSORS=global_settings.TEMPLATE_CONTEXT_PROCESSORS + (
        'djangojs.tests.custom_processor',
    )
)
class JsTests(JsTestCase):
    urls = 'djangojs.test_urls'

    def test_jasmine_suite(self):
        '''It should run its its own Jasmine test suite'''
        self.run_jasmine('djangojs_jasmine', title='Jasmine Test Suite')

    def test_qunit_suite(self):
        '''It should run its its own QUnit test suite'''
        self.run_qunit('djangojs_qunit', title='QUnit Test Suite')


class UrlsJsonViewTest(TestCase):
    urls = 'djangojs.test_urls'

    def setUp(self):
        self.response = self.client.get(reverse('django_js_urls'))
        self.json = json.loads(self.response.content)

    def test_render(self):
        '''It should render a JSON URLs descriptor'''
        self.assertIsNotNone(self.response)
        self.assertEqual(self.response.status_code, 200)
        self.assertEqual(self.response['Content-Type'], 'application/json')
        self.assertIsNotNone(self.json)

    def test_simple_url(self):
        '''It should serialize a simple URL without parameters'''
        self.assertTrue('django_js_urls' in self.json)
        self.assertEqual(self.json['django_js_urls'], '/djangojs/urls')

    def test_url_an_arg(self):
        '''It should serialize an URL with a single anonymous parameter'''
        self.assertTrue('test_arg' in self.json)
        self.assertEqual(self.json['test_arg'], '/test/arg/<>')

    def test_url_many_args(self):
        '''It should serialize an URL with many anonymous parameters'''
        self.assertTrue('test_arg_multi' in self.json)
        self.assertEqual(self.json['test_arg_multi'], '/test/arg/<>/<>')

    def test_url_a_kwarg(self):
        '''It should serialize an URL with a single named parameter'''
        self.assertTrue('test_named' in self.json)
        self.assertEqual(self.json['test_named'], '/test/named/<test>')

    def test_url_many_kwargs(self):
        '''It should serialize an URL with many named parameters'''
        self.assertTrue('test_named_multi' in self.json)
        self.assertEqual(self.json['test_named_multi'], '/test/named/<str>/<num>')

    def test_unnamed_url(self):
        '''It should not serialize unnamed URLs'''
        self.assertFalse('' in self.json)
        for key, value in self.json.iteritems():
            self.assertNotEqual(value, '/test/anonymous')

    def test_optionnal_chars(self):
        '''It should not serialize optionnal characters (take the shortest)'''
        self.assertTrue('opt' in self.json)
        url = self.json['opt']
        self.assertFalse('?' in url)
        self.assertEqual(url, '/test/optionnal')

        self.assertTrue('opt_multi' in self.json)
        url = self.json['opt_multi']
        self.assertFalse('?' in url)
        self.assertEqual(url, '/test/man/optionnal')

    def test_optionnal_groups(self):
        '''It should not serialize optionnal non capturing groups'''
        self.assertTrue('opt_grp' in self.json)
        url = self.json['opt_grp']
        self.assertFalse('?' in url)
        self.assertEqual(url, '/test/optionnal/group')


@override_settings(
    TEMPLATE_CONTEXT_PROCESSORS=global_settings.TEMPLATE_CONTEXT_PROCESSORS + (
        'djangojs.tests.custom_processor',
    )
)
class ContextJsonViewTest(TestCase):
    urls = 'djangojs.test_urls'

    def setUp(self):
        self.response = self.client.get(reverse('django_js_context'))
        self.json = json.loads(self.response.content)

    def test_render(self):
        '''It should render a JSON Context descriptor'''
        self.assertIsNotNone(self.response)
        self.assertEqual(self.response.status_code, 200)
        self.assertEqual(self.response['Content-Type'], 'application/json')
        self.assertIsNotNone(self.json)

    def test_static_url(self):
        '''STATIC_URL should be in context'''
        self.assertTrue('STATIC_URL' in self.json)
        self.assertEqual(self.json['STATIC_URL'], settings.STATIC_URL)

    def test_media_url(self):
        '''MEDIA_URL should be in context'''
        self.assertTrue('MEDIA_URL' in self.json)
        self.assertEqual(self.json['MEDIA_URL'], settings.MEDIA_URL)

    def test_language_code(self):
        '''LANGUAGE_CODE should be in context'''
        self.assertTrue('LANGUAGE_CODE' in self.json)
        self.assertEqual(self.json['LANGUAGE_CODE'], translation.get_language())

    def test_language_bidi(self):
        '''LANGUAGE_BIDI should be in context'''
        self.assertTrue('LANGUAGE_BIDI' in self.json)
        self.assertEqual(self.json['LANGUAGE_BIDI'], translation.get_language_bidi())

    def test_language_name(self):
        '''LANGUAGE_NAME should be in context'''
        self.assertTrue('LANGUAGE_NAME' in self.json)
        code = translation.get_language()
        code = 'en' if code == 'en-us' else code
        language = translation.get_language_info(code)
        self.assertEqual(self.json['LANGUAGE_NAME'], language['name'])

    def test_language_name_local(self):
        '''LANGUAGE_NAME_LOCAL should be in context'''
        self.assertTrue('LANGUAGE_NAME_LOCAL' in self.json)
        code = translation.get_language()
        code = 'en' if code == 'en-us' else code
        language = translation.get_language_info(code)
        self.assertEqual(self.json['LANGUAGE_NAME_LOCAL'], language['name_local'])

    def test_languages(self):
        '''LANGUAGE_BIDI should be in context'''
        self.assertTrue('LANGUAGES' in self.json)
        languages = self.json['LANGUAGES']
        self.assertTrue(isinstance(languages, dict))
        for code, name in settings.LANGUAGES:
            self.assertEqual(languages[code], name)

    def test_any_custom_context_processor(self):
        '''Any custom context processor should be in context'''
        self.assertTrue('CUSTOM' in self.json)
        self.assertEqual(self.json['CUSTOM'], 'CUSTOM_VALUE')


class VerbatimTagTest(TestCase):
    def test_rendering(self):
        '''Should escape {{ and }}'''
        t = Template('''
            {% load js %}
            {% verbatim %}
                <p>{{name}}</p>
                {{{rawname}}}
            {% endverbatim %}
            ''')
        rendered = t.render(Context())

        self.failUnless('{{name}}' in rendered)
        self.failUnless('{{{rawname}}}' in rendered)
        # HTML should not be escaped
        self.failUnless('<p>' in rendered)
        self.failUnless('</p>' in rendered)

    def test_rendering_with_tags(self):
        '''Should process django template tags'''
        t = Template('''
            {% load i18n js %}

            {% verbatim %}
                {% trans "with translation" %}
                {{name}}
                <p>{{{rawname}}}</p>
                {# works with comments too #}
            {% endverbatim %}
            ''')
        rendered = t.render(Context())

        self.failUnless('{{name}}' in rendered)
        self.failUnless('{{{rawname}}}' in rendered)
        self.failUnless('with translation' in rendered)
        # Those should not be rendered :
        self.failUnless('{% trans %}' not in rendered)
        self.failUnless('comments' not in rendered)
        # HTML should not be escaped
        self.failUnless('<p>' in rendered)
        self.failUnless('</p>' in rendered)


class DjangoJsTagTest(TestCase):
    urls = 'djangojs.urls'

    def test_js(self):
        '''Should include static js files'''
        t = Template('''
            {% load js %}
            {% js "js/my.js" %}
            ''')
        rendered = t.render(Context())
        self.failUnless('<script type="text/javascript" src="%sjs/my.js">' % settings.STATIC_URL in rendered)

    def test_javascript(self):
        '''Should include static javascript files'''
        t = Template('''
            {% load js %}
            {% javascript "js/my.js" %}
            ''')
        rendered = t.render(Context())
        self.failUnless('<script type="text/javascript" src="%sjs/my.js">' % settings.STATIC_URL in rendered)

    def test_css(self):
        '''Should include static css files'''
        t = Template('''
            {% load js %}
            {% css "css/my.css" %}
            ''')
        rendered = t.render(Context())
        self.failUnless('<link rel="stylesheet" type="text/css" href="%scss/my.css" />' % settings.STATIC_URL in rendered)

    def test_js_lib(self):
        '''Should include js libraries'''
        t = Template('''
            {% load js %}
            {% js_lib "my-lib.js" %}
            ''')
        rendered = t.render(Context())
        self.failUnless('<script type="text/javascript" src="%sjs/libs/my-lib.js">' % settings.STATIC_URL in rendered)

    def test_jquery_js(self):
        '''Should include jQuery library'''
        t = Template('''
            {% load js %}
            {% jquery_js %}
            ''')
        rendered = t.render(Context())
        self.failUnless('<script type="text/javascript" src="%sjs/libs/jquery-1.8.2.min.js">' % settings.STATIC_URL in rendered)

    def test_django_js(self):
        '''Should include and initialize django.js'''
        t = Template('''
            {% load js %}
            {% django_js %}
            ''')
        rendered = t.render(Context())
        self.failUnless('<script type="text/javascript" src="%sjs/libs/jquery-1.8.2.min.js">' % settings.STATIC_URL in rendered)
        self.failUnless('<script type="text/javascript" src="%sjs/djangojs/django.js">' % settings.STATIC_URL in rendered)
        self.failUnless('window.DJANGO_JS' in rendered)
        self.failUnless('Django.init();' in rendered)

    def test_django_js_init_false(self):
        '''Should include and not initialize django.js'''
        t = Template('''
            {% load js %}
            {% django_js init=false %}
            ''')
        rendered = t.render(Context())
        self.failUnless('<script type="text/javascript" src="%sjs/libs/jquery-1.8.2.min.js">' % settings.STATIC_URL in rendered)
        self.failUnless('<script type="text/javascript" src="%sjs/djangojs/django.js">' % settings.STATIC_URL in rendered)
        self.failUnless('window.DJANGO_JS' in rendered)
        self.failIf('Django.init();' in rendered)

    def test_django_js_jquery_false(self):
        '''Should include django.js without jQuery'''
        t = Template('''
            {% load js %}
            {% django_js jquery=false %}
            ''')
        rendered = t.render(Context())
        self.failIf('<script type="text/javascript" src="%sjs/libs/jquery-1.8.2.min.js">' % settings.STATIC_URL in rendered)
        self.failUnless('<script type="text/javascript" src="%sjs/djangojs/django.js">' % settings.STATIC_URL in rendered)

    def test_django_js_init_jquery_false(self):
        '''Should include unintialized initialize django.js without jQuery'''
        t = Template('''
            {% load js %}
            {% django_js init=false jquery=false %}
            ''')
        rendered = t.render(Context())
        self.failIf('<script type="text/javascript" src="%sjs/libs/jquery-1.8.2.min.js">' % settings.STATIC_URL in rendered)
        self.failUnless('<script type="text/javascript" src="%sjs/djangojs/django.js">' % settings.STATIC_URL in rendered)
        self.failUnless('window.DJANGO_JS' in rendered)
        self.failIf('Django.init();' in rendered)

    def test_django_js_i18n(self):
        '''Should include django.js with i18n support'''
        t = Template('''
            {% load js %}
            {% django_js %}
            ''')
        rendered = t.render(Context())
        self.failUnless('<script type="text/javascript" src="%sjs/libs/jquery-1.8.2.min.js">' % settings.STATIC_URL in rendered)
        self.failUnless('<script type="text/javascript" src="%sjs/djangojs/django.js">' % settings.STATIC_URL in rendered)
        self.failUnless('<script type="text/javascript" src="/trans">' in rendered)
        self.failUnless('window.DJANGO_JS' in rendered)


class JsTestViewTest(TestCase):

    def test_no_js_file(self):
        view = JsTestView()
        self.assertEqual(view.get_js_files(), [])

    def test_single_js_file(self):
        view = JsTestView(js_files='test/js/libs/jasmine-djangojs.js')
        self.assertEqual(view.get_js_files(), ['test/js/libs/jasmine-djangojs.js'])

    def test_multi_js_file(self):
        view = JsTestView(js_files=['test/js/libs/jasmine-djangojs.js', 'test/js/libs/jasmine.js'])

        files = view.get_js_files()

        self.assertIn('test/js/libs/jasmine-djangojs.js', files)
        self.assertIn('test/js/libs/jasmine.js', files)

    def test_single_glob_expression(self):
        view = JsTestView(js_files='test/js/libs/jasmine-*.js')

        files = view.get_js_files()

        self.assertIn('test/js/libs/jasmine-djangojs.js', files)
        self.assertIn('test/js/libs/jasmine-html.js', files)
        self.assertIn('test/js/libs/jasmine-jquery.js', files)
        self.assertNotIn('test/js/libs/jasmine.js', files)

    def test_multi_glob_expression(self):
        view = JsTestView(js_files=['test/js/libs/jasmine-*.js', 'test/js/libs/qunit-*.js'])

        files = view.get_js_files()

        self.assertIn('test/js/libs/jasmine-djangojs.js', files)
        self.assertIn('test/js/libs/jasmine-html.js', files)
        self.assertIn('test/js/libs/jasmine-jquery.js', files)
        self.assertIn('test/js/libs/qunit-tap.js', files)
        self.assertNotIn('test/js/libs/jasmine.js', files)
        self.assertNotIn('test/js/libs/qunit.js', files)



# -*- coding: utf-8 -*-
from django import forms
from django.conf.urls import patterns, url, include
from django.core.urlresolvers import reverse
from django.views.generic.edit import BaseFormView
from django.views.generic import TemplateView

from djangojs.views import JasmineView, QUnitView


class TestForm(forms.Form):
    message = forms.CharField()


class TestFormView(BaseFormView):
    form_class = TestForm

    def get_success_url(self):
        return reverse('opt')


class JasmineTestView(JasmineView):
    template_name = 'djangojs/test/jasmine-runner.html'

    def get_context_data(self, **kwargs):
        context = super(JasmineTestView, self).get_context_data(**kwargs)
        context['form'] = TestForm()
        return context


class QUnitTestView(QUnitView):
    template_name = 'djangojs/test/qunit-runner.html'

    def get_context_data(self, **kwargs):
        context = super(QUnitTestView, self).get_context_data(**kwargs)
        context['form'] = TestForm()
        return context

urlpatterns = patterns('',
    url(r'^djangojs/', include('djangojs.urls')),

    url(r'^jasmine$', JasmineTestView.as_view(), name='djangojs_jasmine'),
    url(r'^qunit$', QUnitTestView.as_view(), name='djangojs_qunit'),

    url(r'^test/form$', TestFormView.as_view(), name='test_form'),
    url(r'^test/arg/(\d+)$', TemplateView.as_view(template_name='djangojs/test/test1.html'), name='test_arg'),
    url(r'^test/arg/(\d+)/(\w)$', TemplateView.as_view(template_name='djangojs/test/test1.html'), name='test_arg_multi'),
    url(r'^test/named/(?P<test>\w+)$', TemplateView.as_view(template_name='djangojs/test/test1.html'), name='test_named'),
    url(r'^test/named/(?P<str>\w+)/(?P<num>\d+)$', TemplateView.as_view(template_name='djangojs/test/test1.html'), name='test_named_multi'),
    url(r'^test/optionnals?$', TemplateView.as_view(template_name='djangojs/test/test1.html'), name="opt"),
    url(r'^test/many?/optionnals?$', TemplateView.as_view(template_name='djangojs/test/test1.html'), name="opt_multi"),
    url(r'^test/optionnal/(?:capturing)?group$', TemplateView.as_view(template_name='djangojs/test/test1.html'), name="opt_grp"),
)
